@@ -27,7 +27,8 @@ sub _options_block {
 #   the --type-add option.
 
 
-# Directories to ignore
+### Directories to ignore
+
 # Bazaar
 --ignore-directory=is:.bzr
 
@@ -80,9 +81,14 @@ sub _options_block {
 # Perl Devel::Cover module's output directory
 --ignore-directory=is:cover_db
 
+# Node modules created by npm
+--ignore-directory=is:node_modules
 
+# CMake cache
+--ignore-directory=is:CMakeFiles
+
+### Files to ignore
 
-# Files to ignore
 # Backup files
 --ignore-file=ext:bak
 --ignore-file=match:/~$/
@@ -97,20 +103,29 @@ sub _options_block {
 --ignore-file=match:/core\.\d+$/
 
 # minified Javascript
---ignore-file=match:/[.]min[.]js$/
+--ignore-file=match:/[.-]min[.]js$/
 --ignore-file=match:/[.]js[.]min$/
 
 # minified CSS
 --ignore-file=match:/[.]min[.]css$/
 --ignore-file=match:/[.]css[.]min$/
 
+# PDFs, because they pass Perl's -T detection
+--ignore-file=ext:pdf
 
-# Filetypes defined
+# Common graphics, just as an optimization
+--ignore-file=ext:gif,jpg,jpeg,png
+
+
+### Filetypes defined
 
 # Perl http://perl.org/
---type-add=perl:ext:pl,pm,pod,t
+--type-add=perl:ext:pl,pm,pod,t,psgi
 --type-add=perl:firstlinematch:/^#!.*\bperl/
 
+# Perl tests
+--type-add=perltest:ext:t
+
 # Makefiles http://www.gnu.org/s/make/
 --type-add=make:ext:mk
 --type-add=make:ext:mak
@@ -156,6 +171,9 @@ sub _options_block {
 # C header files
 --type-add=hh:ext:h
 
+# CoffeeScript http://coffeescript.org/
+--type-add=coffeescript:ext:coffee
+
 # C++
 --type-add=cpp:ext:cpp,cc,cxx,m,hpp,hh,h,hxx
 
@@ -171,6 +189,9 @@ sub _options_block {
 # Delphi http://en.wikipedia.org/wiki/Embarcadero_Delphi
 --type-add=delphi:ext:pas,int,dfm,nfm,dof,dpk,dproj,groupproj,bdsgroup,bdsproj
 
+# Elixir http://elixir-lang.org/
+--type-add=elixir:ext:ex,exs
+
 # Emacs Lisp http://www.gnu.org/software/emacs
 --type-add=elisp:ext:el
 
@@ -201,11 +222,18 @@ sub _options_block {
 # JSP http://www.oracle.com/technetwork/java/javaee/jsp/index.html
 --type-add=jsp:ext:jsp,jspx,jhtm,jhtml
 
+# JSON http://www.json.org/
+--type-add=json:ext:json
+
+# Less http://www.lesscss.org/
+--type-add=less:ext:less
+
 # Common Lisp http://common-lisp.net/
 --type-add=lisp:ext:lisp,lsp
 
 # Lua http://www.lua.org/
 --type-add=lua:ext:lua
+--type-add=lua:firstlinematch:/^#!.*\blua(jit)?/
 
 # Objective-C
 --type-add=objc:ext:m,h
@@ -216,6 +244,9 @@ sub _options_block {
 # OCaml http://caml.inria.fr/
 --type-add=ocaml:ext:ml,mli
 
+# Matlab http://en.wikipedia.org/wiki/MATLAB
+--type-add=matlab:ext:m
+
 # Parrot http://www.parrot.org/
 --type-add=parrot:ext:pir,pasm,pmc,ops,pod,pg,tg
 
@@ -241,6 +272,9 @@ sub _options_block {
 # Rust http://www.rust-lang.org/
 --type-add=rust:ext:rs
 
+# Sass http://sass-lang.com
+--type-add=sass:ext:sass,scss
+
 # Scala http://www.scala-lang.org/
 --type-add=scala:ext:scala
 
