@@ -36,11 +36,12 @@ Finally, ack takes settings from the command line.
 use strict;
 use warnings;
 
+use App::Ack ();
 use App::Ack::ConfigDefault;
 use Cwd 3.00 ();
 use File::Spec 3.00;
 
-use if ($^O =~ /MSWin32/ ? 1 : 0), "Win32";
+use if ($^O eq 'MSWin32'), 'Win32';
 
 =head1 METHODS
 
@@ -50,22 +51,17 @@ Creates a new config finder.
 
 =cut
 
-our $is_win = 0;
-
 sub new {
     my ( $class ) = @_;
 
-    $is_win = $^O =~ /MSWin32/,
-
     return bless {}, $class;
 }
 
 sub _remove_redundancies {
     my ( @configs ) = @_;
 
-    if ( $is_win ) {
-        # inode stat always returns 0 on windows,
-        # so just check filenames
+    if ( $App::Ack::is_windows ) {
+        # inode stat always returns 0 on windows, so just check filenames.
         my (%seen, @uniq);
 
         foreach my $path (@configs) {
@@ -114,15 +110,14 @@ sub _check_for_ackrc {
 
 =head2 $finder->find_config_files
 
-Locates config files, and returns
-a list of them.
+Locates config files, and returns a list of them.
 
 =cut
 
 sub find_config_files {
     my @config_files;
 
-    if( $is_win ) {
+    if ( $App::Ack::is_windows ) {
         push @config_files, map { File::Spec->catfile($_, 'ackrc') } (
             Win32::GetFolderPath(Win32::CSIDL_COMMON_APPDATA()),
             Win32::GetFolderPath(Win32::CSIDL_APPDATA()),
@@ -155,7 +150,7 @@ sub find_config_files {
     return _remove_redundancies( @config_files );
 }
 
-=head2 read_ackrc
+=head2 read_rcfile
 
 Reads the contents of the .ackrc file and returns the arguments.
 
