@@ -5,18 +5,33 @@ use App::Ack ();
 use Cwd ();
 use File::Spec ();
 use File::Temp ();
+use Test::More;
 
 my $orig_wd;
-my @temp_files; # we store temp files here to make sure they're properly
-                # reclaimed at interpreter shutdown
+my @temp_files; # We store temp files here to make sure they're properly reclaimed at interpreter shutdown.
+
+sub check_message {
+    my $msg = shift;
+
+    if ( !$msg ) {
+        my (undef,undef,undef,$sub) = caller(1);
+        Carp::croak( "You must pass a message to $sub" );
+    }
+
+    return $msg;
+}
 
 sub prep_environment {
     delete @ENV{qw( ACK_OPTIONS ACKRC ACK_PAGER HOME )};
     $orig_wd = Cwd::getcwd();
 }
 
-sub is_win32 {
-    return $^O =~ /Win32/;
+sub is_windows {
+    return $^O eq 'MSWin32';
+}
+
+sub is_cygwin {
+    return $^O eq 'cygwin';
 }
 
 sub build_ack_invocation {
@@ -113,10 +128,10 @@ sub reslash_all {
 }
 
 sub run_ack {
-    my @args = @_;
-
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
+    my @args = @_;
+
     my ($stdout, $stderr) = run_ack_with_stderr( @args );
     @args = grep { ref($_) ne 'HASH' } @args;
 
@@ -124,8 +139,7 @@ sub run_ack {
         fail( q{Automatically fail stderr check for TODO tests.} );
     }
     else {
-        is( scalar @{$stderr}, 0, "Should have no output to stderr: ack @args" )
-            or diag( join( "\n", "STDERR:", @{$stderr} ) );
+        is_deeply( $stderr, [], "Should have no output to stderr: ack @args" );
     }
 
     return wantarray ? @{$stdout} : join( "\n", @{$stdout} );
@@ -161,11 +175,9 @@ sub run_cmd {
 
     my ( @stdout, @stderr );
 
-    if (is_win32) {
-        if($options->{'input'}) {
-            die('input options have not been implemented for Win32 yet');
-        }
-# capture stderr & stdout output into these files (only on Win32)
+    if ( is_windows() ) {
+        require Win32::ShellQuote;
+        # Capture stderr & stdout output into these files (only on Win32).
         my $catchout_file = 'stdout.log';
         my $catcherr_file = 'stderr.log';
 
@@ -173,7 +185,12 @@ sub run_cmd {
         open(SAVEERR, ">&STDERR") or die "Can't dup STDERR: $!";
         open(STDOUT, '>', $catchout_file) or die "Can't open $catchout_file: $!";
         open(STDERR, '>', $catcherr_file) or die "Can't open $catcherr_file: $!";
-        system @cmd;
+        my $cmd = Win32::ShellQuote::quote_system_string(@cmd);
+        if ( my $input = $options->{input} ) {
+            my $input_command = Win32::ShellQuote::quote_system_string(@{$input});
+            $cmd = "$input_command | $cmd";
+        }
+        system $cmd;
         close STDOUT;
         close STDERR;
         open(STDOUT, ">&SAVEOUT") or die "Can't restore STDOUT: $!";
@@ -184,7 +201,6 @@ sub run_cmd {
         @stderr = read_file($catcherr_file);
     }
     else {
-
         my ( $stdout_read, $stdout_write );
         my ( $stderr_read, $stderr_write );
 
@@ -244,11 +260,11 @@ sub run_cmd {
 
             if(my $input = $options->{'input'}) {
                 # XXX check error
-                open STDIN, '-|', @$input;
+                open STDIN, '-|', @$input or die "Can't open: $!";
             }
 
-            open STDOUT, '>&', $stdout_write;
-            open STDERR, '>&', $stderr_write;
+            open STDOUT, '>&', $stdout_write or die "Can't open: $!";
+            open STDERR, '>&', $stderr_write or die "Can't open: $!";
 
             exec @cmd;
         }
@@ -325,29 +341,32 @@ sub pipe_into_ack {
 sub lists_match {
     local $Test::Builder::Level = $Test::Builder::Level + 1; ## no critic
 
-    my @actual = @{+shift};
+    my @actual   = @{+shift};
     my @expected = @{+shift};
-    my $msg = shift;
+    my $msg      = check_message( shift );
 
     # Normalize all the paths
     for my $path ( @expected, @actual ) {
         $path = File::Next::reslash( $path ); ## no critic (Variables::ProhibitPackageVars)
     }
 
-    my $ok;
-    my $rc = eval 'use Test::Differences; 1;';
-    if ( $rc ) {
-        $ok = eq_or_diff( [@actual], [@expected], $msg );
-    }
-    else {
-        $ok = is_deeply( [@actual], [@expected], $msg );
-    }
+    return subtest "lists_match( $msg )" => sub {
+        plan tests => 1;
 
-    if ( !$ok ) {
-        diag( explain( actual => [@actual], expected => [@expected] ) );
-    }
+        my $rc = eval 'use Test::Differences; 1;';
+        if ( $rc ) {
+            $ok = eq_or_diff( [@actual], [@expected], $msg );
+        }
+        else {
+            $ok = is_deeply( [@actual], [@expected], $msg );
+        }
 
-    return $ok;
+        if ( !$ok ) {
+            diag( explain( actual => [@actual], expected => [@expected] ) );
+        }
+
+        return $ok;
+    };
 }
 
 sub ack_lists_match {
@@ -355,25 +374,33 @@ sub ack_lists_match {
 
     my $args     = shift;
     my $expected = shift;
-    my $message  = shift;
+    my $message  = check_message( shift );
+
     my @args     = @{$args};
 
     my @results = run_ack( @args );
-    my $ok = lists_match( \@results, $expected, $message );
-    $ok or diag( join( ' ', '$ ack', @args ) );
 
-    return $ok;
+    return subtest "ack_lists_match( $message )" => sub {
+        plan tests => 1;
+
+        my $ok = lists_match( \@results, $expected, $message );
+        $ok or diag( join( ' ', '$ ack', @args ) );
+    };
 }
 
 # Use this one if you don't care about order of the lines
 sub sets_match {
     local $Test::Builder::Level = $Test::Builder::Level + 1; ## no critic
 
-    my @actual = @{+shift};
+    my @actual   = @{+shift};
     my @expected = @{+shift};
-    my $msg = shift;
+    my $msg      = check_message( shift );
 
-    return lists_match( [sort @actual], [sort @expected], $msg );
+    return subtest "sets_match( $msg )" => sub {
+        plan tests => 1;
+
+        return lists_match( [sort @actual], [sort @expected], $msg );
+    };
 }
 
 sub ack_sets_match {
@@ -381,14 +408,18 @@ sub ack_sets_match {
 
     my $args     = shift;
     my $expected = shift;
-    my $message  = shift;
+    my $message  = check_message( shift );
     my @args     = @{$args};
 
-    my @results = run_ack( @args );
-    my $ok = sets_match( \@results, $expected, $message );
-    $ok or diag( join( ' ', '$ ack', @args ) );
+    return subtest "ack_sets_match( $message )" => sub {
+        plan tests => 2;
 
-    return $ok;
+        my @results = run_ack( @args );
+        my $ok = sets_match( \@results, $expected, $message );
+        $ok or diag( join( ' ', '$ ack', @args ) );
+
+        return $ok;
+    };
 }
 
 
@@ -396,8 +427,7 @@ sub record_option_coverage {
     my ( @command_line ) = @_;
 
     return unless $ENV{ACK_OPTION_COVERAGE};
-    return if $ENV{ACK_STANDALONE}; # we don't need to record the second time
-                                    # around
+    return if $ENV{ACK_STANDALONE}; # We don't need to record the second time around.
 
     my $record_options = File::Spec->catfile($orig_wd, 'record-options');
 
@@ -484,9 +514,9 @@ BEGIN {
                 }
                 $slave->set_raw();
 
-                open STDIN,  '<&', $slave->fileno();
-                open STDOUT, '>&', $slave->fileno();
-                open STDERR, '>&', $slave->fileno();
+                open STDIN,  '<&', $slave->fileno() or die "Can't open: $!";
+                open STDOUT, '>&', $slave->fileno() or die "Can't open: $!";
+                open STDERR, '>&', $slave->fileno() or die "Can't open: $!";
 
                 close $slave;
 
